% Przyklad 9.3:
% m-plik: dither.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 9: Przetworniki analogowo-cyfrowe i cyfrowo-analogowe.
% Autorzy rozdzialu: Wojciech Polchlopek, Roman Rumian.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: kwiecien 2014 r.

clc; clear; close all;

b=12;                       % dlugosc slowa w bitach;
q=2^(-(b-1));               % krok kwantyzacji;

wariant=1;                  % wybieramy konfiguracje eksperymentu (wpisujac liczbe: 1 lub 2, lub 3, lub 4);
switch wariant,
    case 1, s=0.5; A=q;     % s - wybor metody kwantyzacji; A - wybor amplitudy sygnalu;
    case 2, s=0.5; A=2^(-14);
    case 3, s=0;   A=q;
    case 4, s=0;   A=2^(-14);
end;    

N=2^12;   n=0:N-1;          % liczba probek i numery probek;     

k=2^8+0.3;                  % numer elementu DFT (dodanie 0.3 daje efekt przecieku);
Fo=k/N;                     % unormowana czestotliwosc sygnalu [Hz/Hz];

x=A*cos(2*pi*n*Fo);         % sygnal dyskretny o czestotliwosci unormowanej Fo;
xq=floor(x/q+s)*q;          % kwantyzacja;
eq0=xq-x;                   % blad kwantyzacji;

w=hamming(N)';              % okno Hamminga (w MATLABie sa tez dostepne inne okna...);
%w=ones(1,N);               % okno prostokatne (czyli w istocie - brak okna):
                            %      aby zmienic okno nalezy przeniesc znak komentarza 
                            %      na poczatek linijki poprzedzajacej;
                            
X=fft(x.*w)/(N/2);     AX_dB=20*log10(abs(X)+eps);   % widmo amplitudowe DFT w dB;
Xq=fft(xq.*w)/(N/2);   AXq_dB=20*log10(abs(Xq)+eps); % widmo amplitudowe DFT w dB;

% "dither" prostokatny:
d0=rand(1,N);
d0=d0-mean(d0);
d0=d0./max(abs(d0));        % normalizacja;
d1=d0*q/2;                  % dopasowanie amplitudy "szumu dither" do kroku kwantyzacji;
xd1=x+d1;                   % dodanie "szumu dither" do sygnalu;
xd1q=floor(xd1/q+s)*q;      % kwantyzacja; 

eq1=xd1q-x;                 % blad kwantyzacji;
Xd1=fft(xd1.*w)/(N/2);     AXd1_dB=20*log10(abs(Xd1)+eps);  % widmo amplitudowe DFT w dB;
Xd1q=fft(xd1q.*w)/(N/2);   AXd1q_dB=20*log10(abs(Xd1q)+eps);% widmo amplitudowe DFT w dB;
D1=fft(d1.*w)/(N/2);       AD1_dB=20*log10(abs(D1)+eps);    % widmo amplitudowe DFT w dB;
                               
% "dither" trojkatny:
d2=rand(1,N);
d2=d2-mean(d2);
d2=d2./max(abs(d2));        % normalizacja;
d2=(d2+d0)*q/4;             % amplituda "szumu dither";
xd2=x+d2;                   % dodanie "szumu dither" do sygnalu;
xd2q=floor(xd2/q+s)*q;      % kwantyzacja; 

eq2=xd2q-x;                 % blad kwantyzacji;
Xd2=fft(xd2.*w)/(N/2);     AXd2_dB=20*log10(abs(Xd2)+eps);  % widmo amplitudowe DFT w dB;
Xd2q=fft(xd2q.*w)/(N/2);   AXd2q_dB=20*log10(abs(Xd2q)+eps);% widmo amplitudowe DFT w dB;
D2=fft(d2.*w)/(N/2);       AD2_dB=20*log10(abs(D2)+eps);    % widmo amplitudowe DFT w dB;

% Konfrontacja SNR liczonego "calosciowo" z sygnalow (dziwne? - ciekawe dlaczego?):
SNR_xq     = 10*log10(sum(x.^2)/sum(eq0.^2)),
SNR_xd1q   = 10*log10(sum(x.^2)/sum(eq1.^2)),
SNR_xd2q   = 10*log10(sum(x.^2)/sum(eq2.^2)),

[hd1, bd1]=hist(d1,20);     % wyznaczamy histogram (czyli rozklad z proby losowej) dla "dithera" prostokatnego;
[hd2, bd2]=hist(d2,20);     % wyznaczamy histogram (czyli rozklad z proby losowej) dla "dithera" trojkatnego;

F=(0:N-1)/N;                % opis osi dla wykresow widm w czestotliwosci unormowanej [Hz/Hz];
m=0:99;                     % indeksy wybierajace fragmenty sygnalu do wykresow;
axis_v=[0,0.5, -150, -70];  % parametry dla funkcji "axis";

    figure(1);  
        subplot(2,1,1);
            plot(m,x(m+1),'bo-');     hold on;  grid on;
            plot(m,xq(m+1),'rs-');
                    title('a) efekt (re)kwantyzacji sygnalu x[n]');    xlabel('n');  
                    legend('x[n]','po kwantyzacji',4);
        subplot(2,1,2);
            plot(F,AX_dB,'b','linewidth',2);     hold on; grid on;
            plot(F,AXq_dB,'r');
                    title('b) widmo amplitudowe przed i po kwantyzacji');  
                    xlabel('F'); ylabel('Amplituda [dB]')
                    axis(axis_v);  
                    legend('DFT/(N/2) dla x[n]','DFT/(N/2) po kwantyzacji',4);

    figure(2);
        subplot(2,1,1);
            plot(m,x(m+1),'c.-');      hold on;  grid on;
            plot(m,xd1(m+1),'bo-');
            plot(m,xd1q(m+1),'rs-');
                    title('             a) efekt (re)kwantyzacji sygnalu x[n] z dodanym ditherem prostokatnym');
                    xlabel('n'); legend('x[n]','x[n]+d1[n]','po kwantyzacji',4);
        subplot(2,1,2);
            plot(F,AXd1_dB,'b','linewidth',2);     hold on; grid on;
            plot(F,AXd1q_dB,'r');
                    title('b) widmo amplitudowe przed i po kwantyzacji');  xlabel('F'); ylabel('Amplituda [dB]')
                    legend('DFT/(N/2) dla x[n]+d1[n]','DFT/(N/2) po kwantyzacji',4);
                    axis(axis_v); 
                    
    figure(3);
        subplot(2,1,1);
            plot(m,x(m+1),'c.-');      hold on;  grid on;
            plot(m,xd2(m+1),'bo-');
            plot(m,xd2q(m+1),'rs-');
                    title('        a) efekt (re)kwantyzacji sygnalu x[n] z dodanym ditherem trojkatnym');
                    xlabel('n'); legend('x[n]','x[n]+d2[n]','po kwantyzacji',4);
        subplot(2,1,2);
            plot(F,AXd2_dB,'b','linewidth',2);     hold on; grid on;
            plot(F,AXd2q_dB,'r');
                    title('b) widmo amplitudowe przed i po kwantyzacji');  xlabel('F'); ylabel('Amplituda [dB]')
                    legend('DFT/(N/2) dla x[n]+d2[n]','DFT/(N/2) po kwantyzacji',4);
                    axis(axis_v); 
                
% Studium "szumu dither":
    figure(4); % Unormowane (w pionie i w poziomie) histogramy "szumu dither";
        subplot(2,2,1);
            bar(bd1/q,hd1/sum(hd1),1);  grid on;
                    title('Histogram "dithera" prostokatnego');
        subplot(2,2,3);
            plot(F,AD1_dB);             grid on, 
                    title('DFT  realizacji prostokatnego "szumu dither"');
                    xlabel('F');         ylabel('Amplituda [dB]');
                    axis(axis_v); 
        subplot(2,2,2);
            bar(bd2/q,hd2/sum(hd2),1);  grid on;
                    title('Histogram "dithera" trojkatnego');
        subplot(2,2,4);
            plot(F,AD2_dB);             grid on;
                    title('DFT realizacji trojkatnego "szumu dither"');
                    xlabel('F');        ylabel('Amplituda [dB]');
                    axis(axis_v);
                    
% KONIEC PRZYKLADU 9.3.