% Przyklad 9.1:
% m-plik: przyklad_9_1.m
%
% Cyfrowe przetwarzanie sygnalow. Podstawy, multimedia, transmisja. PWN, Warszawa, 2014.
% Rozdzial 9: Przetworniki analogowo-cyfrowe i cyfrowo-analogowe.
% Autorzy rozdzialu: Wojciech Polchlopek, Roman Rumian.
%
% Wiecej na stronie internetowej: http://teledsp.kt.agh.edu.pl
%
% Opracowanie przykladu: marzec 2014 r.

clc; clear; close all;

b=3;         q=2^(-(b-1));  % b - liczba bitow przetwornika A/C, q - krok kwantyzacji;
N=2^8;       fo=440;        % dlugosc ciagu x[n] oraz zalozona czestotliwosc sygnalu w Hz;     
fpr=11025;                  % czestotliwosc probkowania w Hz;
Fo=fo/fpr;   n=0:N-1;       % czestotliowosc unormowana i numery probek;

x=cos(2*pi*Fo*n);           % tworzymy ciag probek o zalozonych parametrach;
x1=x;
x1(x+eps>1-q)=1-q;          % korekta ze wzgledu na asymetrie zakresow dla wartosci dodatnich i ujemnych;
xqr=floor((x1/q)+0.5)*q;    % kwantyzacja z zaokraglaniem (r=rounding);
xqt=floor(x1/q)*q;          % kwantyzacja z obcieciem (t=truncation);

eqr=xqr-x;                  % blad kwantyzacji z zaokragleniem;
eqt=xqt-x;                  % blad kwantyzacji z obcieciem;

w=hamming(N)';
X=fft(x.*w)/(N/2);      AX_dB=20*log10(abs(X)+eps);   % widmo amplitudowe DFT, w dB;
X1=fft(xqt.*w)/(N/2);   AX1_dB=20*log10(abs(X1)+eps); % widmo amplitudowe DFT, w dB;
X2=fft(xqr.*w)/(N/2);   AX2_dB=20*log10(abs(X2)+eps); % widmo amplitudowe DFT, w dB;

% test funkcji przejscia dla obu sposobow kwantyzacji:
x_test=-1:0.0001:1;  
x1_test=x_test;
x1_test(x_test+eps>1-q)=1-q;     % korekta ze wzgledu na asymetrie zakresow dla wartosci dodatnich i ujemnych;
xqr_test=floor((x1_test/q)+0.5)*q;   eqr_test=xqr_test-x_test;
xqt_test=floor((x1_test/q))*q;       eqt_test=xqt_test-x_test;

F=(0:(N-1))/(N); % opis osi dla wykresow widm w czestotliwosci unormowanej [Hz/Hz];
   
    figure(1);
        subplot(2,1,1);
            plot(x_test,x_test,'k.-');    grid on;    hold on;
            plot(x_test,xqr_test,'ro-','markersize',5,'markerface','r');
            plot(x_test,xqt_test,'b.-');
                title('Porownanie funkcji przejscia dla dwoch metod kwantyzacji');
                legend('bez zmian','kwantyzacja z zaokraglaniem','kwantyzacja z obcieciem',4);
        subplot(2,1,2);
            plot(x_test,eqr_test,'ro-','markersize',5,'markerface','r');    grid on;    hold on;
            plot(x_test,eqt_test,'b.-');
                title('Porownanie bledu kwantyzacji');
                legend('z zaokraglaniem','z obcieciem',1);
    figure(2); n=0:64;  ms=5;
        plot(n, x(n+1),'bo-','markersize',ms);      grid on;    hold on;
        plot(n, xqr(n+1),'rs-','markersize',ms);
        plot(n, eqr(n+1),'kv-','markersize',ms);
                title('Kwantyzacja z zaokraglaniem');
                xlabel('n');        ylabel('x[n],  x_q[n],  e_q[n]');
                legend('sygnal x[n]','sygnal skwantowany','wartosci bledu kwantyzacji',0);
                    xlim([0,max(n)]);
    figure(3);
        plot(n, x(n+1),'bo-','markersize',ms);      grid on;    hold on;
        plot(n, xqt(n+1),'rs-','markersize',ms);
        plot(n, eqt(n+1),'kv-','markersize',ms);
                title('Kwantyzacja z obcieciem');
                xlabel('n');        ylabel('x[n],  x_q[n],  e_q[n]');
                legend('sygnal x[n]','sygnal skwantowany','wartosci bledu kwantyzacji',0);
                    xlim([0,max(n)]);
    figure(4);
        plot(F,AX_dB,'bo-','markersize',4);         grid on;    hold on;
        plot(F,AX1_dB,'ro-','markersize',5);
        plot(F,AX2_dB,'ks-','markersize',2);
                title('Porownanie widm amplitudowych DFT');
                xlabel('F [Hz/Hz]'); ylabel('Amplituda [dB]');
                legend('dla sygnalu x[n]','po kwantyzacji z zaokragleniem','po kwantyzacji z obcieciem',1);
                    axis([0,0.5,-80,0]);
        
% KONIEC PRZYKLADU 9.1.